#include <stdio.h>
#include <xview/xview.h>
#include <sys/stat.h>
#include "tool.h"
#include "g.h"

static int dx;
static int dy;

#include "menu.h"
#include "ext.h"

drawbox(x1, y1)
int x1, y1;
{
	draw_vector(fix_x + x1, fix_y + y1, cur_x + x1, fix_y + y1);
	draw_vector(cur_x + x1, fix_y + y1, cur_x + x1, cur_y + y1);
	draw_vector(cur_x + x1, cur_y + y1, fix_x + x1, cur_y + y1);
	draw_vector(fix_x + x1, cur_y + y1, fix_x + x1, fix_y + y1);
}

erase_box(x, y)
int x, y;
{
	drawbox(x, y);
}

insert_node_list(n)
struct g_node *n;
{
	struct NODE_LIST *nl;

	if (n == NULL) return;
	nl = (struct NODE_LIST *) malloc(sizeof(struct NODE_LIST));
	nl->node = n;
	nl->next = NULL;
	if (node_list != NULL) nl->next = node_list;
	node_list = nl;
}

release_node_list()
{
	struct NODE_LIST *nl, *b;

	for (nl = node_list; nl != NULL; b = nl, nl = nl->next, free(b));
	node_list = NULL;
}

insert_link_list(l)
struct g_link *l;
{
	struct LINK_LIST *nl;

	if (l == NULL) return;
	for (nl = link_list; nl != NULL; nl = nl->next) 
		if (nl->link == l) return;
	nl = (struct LINK_LIST *) malloc(sizeof(struct LINK_LIST));
	nl->link = l;
	nl->next = NULL;
	if (link_list != NULL) nl->next = link_list;
	link_list = nl;
}

release_link_list()
{
	struct LINK_LIST *nl, *b;

	for (nl = link_list; nl != NULL; b = nl, nl = nl->next, free(b));
	link_list = NULL;
}

insert_rspt_list(n)
struct g_rspt *n;
{
	struct RSPT_LIST *nl;

	if (n == NULL) return;
	nl = (struct RSPT_LIST *) malloc(sizeof(struct RSPT_LIST));
	nl->rspt = n;
	nl->next = NULL;
	if (rspt_list != NULL) nl->next = rspt_list;
	rspt_list = nl;
}

release_rspt_list()
{
	struct RSPT_LIST *nl, *b;

	for (nl = rspt_list; nl != NULL; b = nl, nl = nl->next, free(b));
	rspt_list = NULL;
}

insert_label_list(n)
struct g_label *n;
{
	struct LABEL_LIST *nl;

	if (n == NULL) return;
	nl = (struct LABEL_LIST *) malloc(sizeof(struct LABEL_LIST));
	nl->label = n;
	nl->next = NULL;
	if (label_list != NULL) nl->next = label_list;
	label_list = nl;
}

release_label_list()
{
	struct LABEL_LIST *nl, *b;

	for (nl = label_list; nl != NULL; b = nl, nl = nl->next, free(b));
	label_list = NULL;
}

insert_its_links_to_list(n)
struct g_node *n;
{
	struct g_link *link;

	if (n == NULL) return;
	if (n == g->initial_node) {
		erase_initial_node_symbol(n);
		erase_machine_name();
		}
	erase_node(n->x, n->y);
	erase_node_label(n);
	insert_node_list(n);
	for (link = n->inlink; link != NULL; link = link->inlink) 
		insert_link_list(link);
	for (link = n->outlink; link != NULL; link = link->outlink) 
		insert_link_list(link);
}

in_node_list(n)
struct g_node *n;
{
	struct NODE_LIST *nl;

	if (n == NULL) return(0);
	for (nl = node_list; nl != NULL; nl = nl->next) 
		if (nl->node == n) return(1);
	return(0);
}

its_node_in_node_list(l)
struct g_link *l;
{
	if (l == NULL) return(0);
	if (in_node_list(l->src_port.node) || in_node_list(l->dst_port.node))
		return(1);
	else return(0);
}

release_list()
{
	struct NODE_LIST *nl, *bn;
	struct LINK_LIST *ll, *bl;
	struct RSPT_LIST *rl, *br;
	struct LABEL_LIST *lal, *bla;

	for (nl = node_list; nl != NULL; bn = nl, nl = nl->next, free(bn));
	node_list = NULL;
	for (ll = link_list; ll != NULL; bl = ll, ll = ll->next, free(bl));
	link_list = NULL;
	for (rl = rspt_list; rl != NULL; br = rl, rl = rl->next, free(br));
	rspt_list = NULL;
	for (lal = label_list; lal != NULL; bla = lal, lal = lal->next, free(bla));
	label_list = NULL;
}

remove_rspt_and_link(l)
struct g_link *l;
{
	struct g_rspt *f, *b;
	struct g_link *link, *prev;

	for (f = l->rest_points; f != NULL; b = f, f = f->next_inlink, delete_rspt(b), free(b));
	delete_link(l);
	if (l->label != NULL) { delete_label(l->label); free(l->label);}
	remove_link_from_inlink_and_outlink(l);
	free(l);
}

remove_sub_topology()
{

	struct LINK_LIST *ll;
	struct NODE_LIST *nl;
	int x1, y1, x2, y2;
	int change_initial_node;

	if (cur_x > fix_x) {
		x1 = fix_x;
		x2 = cur_x;
		}
	else {
		x1 = cur_x;
		x2 = fix_x;
		}
	if (cur_y > fix_y) {
		y1 = fix_y;
		y2 = cur_y;
		}
	else {
		y1 = cur_y;
		y2 = fix_y;
		}

	node_list = NULL;
	link_list = NULL;
	rspt_list = NULL;
	label_list = NULL;

	for (node = g->first_node; node != NULL; node = node->next) 
		if (node->x >= x1 && node->x <= x2 &&
		    node->y >= y1 && node->y <= y2)
			insert_its_links_to_list(node);	
	for (ll = link_list; ll != NULL; ll = ll->next) 
		if (in_node_list(ll->link->src_port.node) || 
			in_node_list(ll->link->dst_port.node)) 
			erase_link(ll->link);
	for (ll = link_list; ll != NULL; ll = ll->next) 
		if (in_node_list(ll->link->src_port.node) || 
			in_node_list(ll->link->dst_port.node)) {
			erase_edge_label(ll->link->label);
			remove_rspt_and_link(ll->link);
			}
	change_initial_node = 0;
	for (nl = node_list; nl != NULL; nl = nl->next) {
		node = nl->node;
		if (node == g->initial_node) change_initial_node = 1;
		delete_node(node);
		free_node(node);
		}
	if (change_initial_node == 1) {
		g->initial_node = g->first_node; 
		draw_initial_node_symbol(g->initial_node);
		draw_machine_name();
		}
	release_list();
	erase_box(0, 0);
}

save_sub_topology()
{

	struct LINK_LIST *ll;
	struct NODE_LIST *nl;
	int x1, y1, x2, y2;

	if (cur_x > fix_x) {
		x1 = fix_x;
		x2 = cur_x;
		}
	else {
		x1 = cur_x;
		x2 = fix_x;
		}
	if (cur_y > fix_y) {
		y1 = fix_y;
		y2 = cur_y;
		}
	else {
		y1 = cur_y;
		y2 = fix_y;
		}

	node_list = NULL;
	link_list = NULL;
	rspt_list = NULL;
	label_list = NULL;

	for (node = g->first_node; node != NULL; node = node->next) 
		if (node->x >= x1 && node->x <= x2 &&
		    node->y >= y1 && node->y <= y2)
			insert_node_list(node);	
	for (nl = node_list; nl != NULL; nl = nl->next) {
		node = nl->node;
		for (link = node->inlink; link != NULL; link = link->inlink) 
			if (in_node_list(link->src_port.node) && 
			    in_node_list(link->dst_port.node)) 
			insert_link_list(link);
		}
	for (ll = link_list; ll != NULL; ll = ll->next) {
		for (rspt = ll->link->rest_points; rspt != NULL; rspt = rspt->next_inlink) 
			insert_rspt_list(rspt);
		insert_label_list(ll->link->label);
		}
	store_sub_topology();
	release_list();
	erase_box(0, 0);
}

in_label_list(label)
struct g_label *label;
{
	struct LABEL_LIST *la;

	if (label == NULL) return(0);
	for (la = label_list; la != NULL; la = la->next) 
		if (la->label == label) return(1);
	return(0);
}

draw_sub_topology(x, y)
int x, y;
{
	struct g_link *link;
	struct NODE_LIST *nl;
	struct LINK_LIST *ll;
	struct RSPT_LIST *rl;
	struct LABEL_LIST *lal;
	point s, d, p;

	/* adjust the position of objects in the sub-topology */
	for (nl = node_list; nl != NULL; nl = nl->next) {
		node = nl->node;
		delete_node(node);
		node->x += x;
		node->y += y;
		node->next = NULL;
		insert_node(node);
		}
	for (rl = rspt_list; rl != NULL; rl = rl->next) {
		if (its_node_in_node_list(rl->rspt->link)) {
	 		rspt = rl->rspt;
	 		delete_rspt(rspt);
	 		rspt->x += x;
	 		rspt->y += y;
	 		rspt->next = NULL;
	 		insert_rspt(rspt);
			}
		}
	for (lal = label_list; lal != NULL; lal = lal->next) {
		if (its_node_in_node_list(lal->label->link)) {
			label = lal->label;
			delete_label(label);
			label->x += x;
			label->y += y;
			label->next = NULL;
			insert_label(label);
			}
		}
	for (ll = link_list; ll != NULL; ll = ll->next) {
		link = ll->link;
		if (in_node_list(link->src_port.node)) {
			s.x = link->src_port.node->x;
			s.y = link->src_port.node->y;
			if (link->rest_points == NULL) {
				d.x = link->dst_port.node->x;
				d.y = link->dst_port.node->y;
				find_port(d, s, &p);
				link->src_port.x = p.x;
				link->src_port.y = p.y;
				find_port(s, d, &p);
				link->dst_port.x = p.x;
				link->dst_port.y = p.y;
			}
			else {
				d.x = link->rest_points->x;
				d.y = link->rest_points->y;
				find_port(d, s, &p);
				link->src_port.x = p.x;
				link->src_port.y = p.y;
				}
			}
		if (in_node_list(link->dst_port.node)) {
			d.x = link->dst_port.node->x;
			d.y = link->dst_port.node->y;
			if (link->rest_points == NULL) {
				s.x = link->src_port.node->x;
				s.y = link->src_port.node->y;
				find_port(s, d, &p);
				link->dst_port.x = p.x;
				link->dst_port.y = p.y;
				find_port(d, s, &p);
				link->src_port.x = p.x;
				link->src_port.y = p.y;
				}
			else {
				rspt = last_restpt(link->rest_points);
				s.x = rspt->x;
				s.y = rspt->y;
				find_port(s, d, &p);
				link->dst_port.x = p.x;
				link->dst_port.y = p.y;
				}
			}
		}
	for (nl = node_list; nl != NULL; nl = nl->next) {
		draw_node(nl->node->x, nl->node->y);
		write_node_label(nl->node);
		if (nl->node == g->initial_node) {
			draw_initial_node_symbol(nl->node);
			draw_machine_name();
			}
		}
	for (ll = link_list; ll != NULL; ll = ll->next) 
		draw_link(ll->link);
	for (lal = label_list; lal != NULL; lal = lal->next) 
		write_edge_label(lal->label);
	for (ll = link_list; ll != NULL; ll = ll->next) 
		if (in_label_list(ll->link->label)) continue;
		else write_edge_label(ll->link->label);
	release_list();
}

erase_sub_topology()
{
	struct LINK_LIST *ll;
	int x1, y1, x2, y2;

	if (cur_x > fix_x) {
		x1 = fix_x;
		x2 = cur_x;
		}
	else {
		x1 = cur_x;
		x2 = fix_x;
		}
	if (cur_y > fix_y) {
		y1 = fix_y;
		y2 = cur_y;
		}
	else {
		y1 = cur_y;
		y2 = fix_y;
		}

	node_list = NULL;
	link_list = NULL;
	rspt_list = NULL;
	label_list = NULL;

	for (node = g->first_node; node != NULL; node = node->next) 
		if (node->x >= x1 && node->x <= x2 &&
		    node->y >= y1 && node->y <= y2)
			insert_its_links_to_list(node);	
	for (rspt = g->first_rspt; rspt != NULL; rspt = rspt->next)
		if (rspt->x >= x1 && rspt->x <= x2 &&
		    rspt->y >= y1 && rspt->y <= y2) {
			insert_link_list(rspt->link);
			insert_rspt_list(rspt);
			}
	for (label = g->first_label; label != NULL; label = label->next)
		if (label->x >= x1 && label->x <= x2 &&
		    label->y >= y1 && label->y <= y2) {
			insert_link_list(label->link); 
			insert_label_list(label);
			}
	for (ll = link_list; ll != NULL; ll = ll->next) {
		erase_link(ll->link);
		}
	for (ll = link_list; ll != NULL; ll = ll->next) {
		erase_edge_label(ll->link->label);
		}
}

move_area_reader(ie)
Event *ie;
{

	if (zoom == 0) {
		ie->ie_locx = ie->ie_locx/2 - DX;
		ie->ie_locy = ie->ie_locy/2 - DY;
	}
	else {
		ie->ie_locx = ie->ie_locx*zoom - DX;
		ie->ie_locy = ie->ie_locy*zoom - DY;
	}
	if (win_inputnegevent(ie) ) return;

	switch (ie->ie_code) {
	case LOC_MOVE :
		erase_box(dx, dy);
		dx = ie->ie_locx - cur_x;
		dy = ie->ie_locy - cur_y;
		drawbox(dx, dy);
		return;
	case CNTL_C :
		erase_box(dx, dy);
		dx = 0;
		dy = 0;
		draw_sub_topology(dx, dy);
		put_text("Please point to the upper left conner of the sub-topology to be moved.");
		reset_canvas_reader();
		return;
	case MS_MIDDLE :
		erase_box(dx, dy);
		draw_sub_topology(dx, dy);
		put_text("Please point to the upper left conner of the sub-topology to be moved.");
		reset_canvas_reader();
		return;
		}
}

init_area_reader(ie)
Event *ie;
{

	fix_x = ie->ie_locx;
	fix_y = ie->ie_locy;
	cur_x = fix_x;
	cur_y = fix_y;
	drawbox(0, 0);
	put_text_high("Now point to the opposite corner;");
	put_text("  hit middle button to fix the area; CNTL_C to cancel the operation.");
}

area_reader(ie)
Event *ie;
{


	if (zoom == 0) {
		ie->ie_locx = ie->ie_locx/2 - DX;
		ie->ie_locy = ie->ie_locy/2 - DY;
	}
	else {
		ie->ie_locx = ie->ie_locx*zoom - DX;
		ie->ie_locy = ie->ie_locy*zoom - DY;
	}
	if (win_inputnegevent(ie) ) return;

	switch (ie->ie_code) {
	case LOC_MOVE :
		erase_box(0, 0);
		cur_x = ie->ie_locx;
		cur_y = ie->ie_locy;
		drawbox(0, 0);
		return;
	case MS_MIDDLE :
	   if (mode == MOVE_SUB_GRAPH) {
		erase_sub_topology();
		dx = 0;
		dy = 0;
		put_text_high("");
		put_text("Now move the sub-topology to new position.");
		canvas_reader = move_area_reader;
		return;
	   } else if (mode == REMOVE_SUB_GRAPH) {
		remove_sub_topology();
		put_text_high("");
		put_text("Please point to the upper left corner of the sub-topology.");
		reset_canvas_reader();
		return;
	   } else if (mode == SAVE_SUB_GRAPH) {
		save_sub_topology();
		sprintf(buf, "%s is saveed!", file_name);
		put_text_high(buf);
		put_text("");
		sleep(1);
		put_text_high("");
		put_text("Please select the command in the middle button menu.");
		mode = 999;
		reset_canvas_reader();
		return;
		}
		return;
	case CNTL_C :
		mode = 999;
		erase_box(0, 0);
		reset_canvas_reader();
		put_text_high("");
		put_text("Please select the command in the middle button menu.");
		return;
		}
}
