#include <stdio.h>
#include <math.h>

snhcsh (sinhm,coshm,x,isw)
int isw;
double *sinhm,*coshm,x;
{
      double expx, sp2,sp1,sq1,cp2,cp1,cq1,zp2,zp1,zq1,zq2,xx,ax,xs;

      sp2 = 5.04850926418006e-04;
      sp1 = 3.62841692246321e-02;
      sq1 = -1.37157937097122e-02;
      cp2 = 1.31625490355985e-03;
      cp1 = 6.57866547762733e-02;
      cq1 = -1.75465241841312e-02;
      zp2 = 1.40048186158693e-04;
      zp1 = 1.67309141907440e-02;
      zq2 = 9.82154460147143e-05;
      zq1 = -1.66024148976133e-02;
      xx = x;
      ax = fabs(xx);
      xs = xx*xx;
      if ((ax >= 2.20) || (ax >= 1.17 && isw != 2)) expx = exp(ax);
/*
 sinhm approximation;
*/
      if (isw == 1 || isw == 2) goto L2;
      if (ax >= 1.17) goto L1;
      *sinhm = (((sp2*xs+sp1)*xs+1.0)*xs*xx)/((sq1*xs+1.0)*6.0);
      goto L2;
  L1: *sinhm = (expx-1.0/expx)/2.0-ax;
      if (xx < 0.0) *sinhm = -*sinhm;
/*
 coshm approximation;
*/
  L2: if (isw != 0 && isw != 1) goto L4;
      if (ax >= 1.17) goto L3;
      *coshm = (((cp2*xs+cp1)*xs+1.0)*xs)/((cq1*xs+1.0)*2.0);
      goto L4;
  L3: *coshm = (expx+1.0/expx)/2.0-1.0;
/*
 coshmm approximation;
*/
  L4: if (isw <= 1) return;
      if (ax >= 2.20) goto L5;
      *coshm = (((zp2*xs+zp1)*xs+1.0)*xs*xs)/(((zq2*xs+zq1)*xs+1.0)*24.0);
      return;
  L5: *coshm = (expx+1.0/expx)/2.0-1.0-xs/2.0;
      return;
}


double
sinhmd(x)
double x;
{
      double si, dummy;

      snhcsh(&si, &dummy, x, -1);
      return((double) si) ;
}

kurv1 (n,x,y,slp1,slpn,islpsw,xp,yp,temp,s, sigma, ierr)
int n, islpsw, ierr;
double x[],y[],slp1,slpn,xp[],yp[],temp[],s[],sigma;
{
      int i, nm1, np1, im1, ibak;
      double sigmap, slpp1x, slpp1y, dels1, dels2,
	      c1, c2, c3, delt, sx, sy, slppnx, slppny,
	      delsn, delsnm, dx1, dy1, diag1, sdiag1,
	      dx2, dy2, diag2, sdiag2, diag, diagin;

      nm1 = n - 1;
      np1 = n + 1;
      ierr = 0;
      if (n <= 1) goto L11;
/*
c determine polygonal arclengths
*/
      s[1] = 0.0;
      for (i=2; i <= n; i++) {
	im1 = i-1;
        s[i] = s[im1]+sqrt((x[i]-x[im1])*(x[i]-x[im1])+(y[i]-y[im1])*(y[i]-y[im1]));

	}
/*
c denormalize tension factor
*/
      sigmap = fabs(sigma)*((double)(n-1))/s[n];
/*
c approximate end slopes
*/
      if (islpsw >= 2) goto L2;
      slpp1x = cos(slp1);
      slpp1y = sin(slp1);
      goto L4;
  L2: dels1 = s[2]-s[1];
      dels2 = dels1+dels1;
      if (n > 2) dels2 = s[3]-s[1];
      if (dels1 == 0.0 || dels2 == 0.0) goto L12;
      ceez(dels1,dels2,sigmap,&c1,&c2,&c3,n);
      sx = c1*x[1]+c2*x[2];
      sy = c1*y[1]+c2*y[2];
      if (n == 2) goto L3;
      sx = sx+c3*x[3];
      sy = sy+c3*y[3];
  L3: delt = sqrt(sx*sx+sy*sy);
      slpp1x = sx/delt;
      slpp1y = sy/delt;
  L4: if (islpsw == 1 || islpsw == 3) goto L5;
      slppnx = cos(slpn);
      slppny = sin(slpn);
      goto L7;
  L5: delsn = s[n]-s[nm1];
      delsnm = delsn+delsn;
      if (n > 2) delsnm = s[n]-s[n-2];
      if (delsn == 0. || delsnm == 0.) goto L12;
      ceez (-delsn,-delsnm,sigmap,&c1,&c2,&c3,n);
      sx = c1*x[n]+c2*x[nm1];
      sy = c1*y[n]+c2*y[nm1];
      if (n == 2) goto L6;
      sx = sx+c3*x[n-2];
      sy = sy+c3*y[n-2];
  L6: delt = sqrt(sx*sx+sy*sy);
      slppnx = sx/delt;
      slppny = sy/delt;
/*
 set up right hand sides and tridiagonal system for xp and;
 yp and perform forward elimination;
*/
  L7: dx1 = (x[2]-x[1])/s[2];
      dy1 = (y[2]-y[1])/s[2];
      terms (&diag1, &sdiag1, sigmap,s[2]);
      xp[1] = (dx1-slpp1x)/diag1;
      yp[1] = (dy1-slpp1y)/diag1;
      temp[1] = sdiag1/diag1;
      if (n == 2) goto L9;
      for (i=2; i <= nm1; i++) {
	 dels2 = s[i+1]-s[i];
	 if (dels2 == 0.0) goto L12;
	 dx2 = (x[i+1]-x[i])/dels2;
	 dy2 = (y[i+1]-y[i])/dels2;
	 terms (&diag2, &sdiag2,sigmap, dels2);
	 diag = diag1+diag2-sdiag1*temp[i-1];
	 diagin = 1./diag;
	 xp[i] = (dx2-dx1-sdiag1*xp[i-1])*diagin;
	 yp[i] = (dy2-dy1-sdiag1*yp[i-1])*diagin;
	 temp[i] = sdiag2*diagin;
	 dx1 = dx2;
	 dy1 = dy2;
	 diag1 = diag2;
         sdiag1 =sdiag2;
	 }
  L9: diag = diag1-sdiag1*temp[nm1];
      xp[n] = (slppnx-dx1-sdiag1*xp[nm1])/diag;
      yp[n] = (slppny-dy1-sdiag1*yp[nm1])/diag;
/*
 perform back substitution;
*/
      for (i=2; i <= n; i++) {
	ibak = np1-i;
	xp[ibak] = xp[ibak]-temp[ibak]*xp[ibak+1];
        yp[ibak] = yp[ibak]-temp[ibak]*yp[ibak+1];
	}
      return;
/*
 top few points;
*/
 L11: ierr = 1;
      return;
/*
 coincident adjacent points;
*/
 L12: ierr = 2;
      return;
}

kurv2(t,xs,ys,n,x,y,xp,yp,s,sigma)
int n;
double t,*xs,*ys,x[],y[],xp[],yp[],s[],sigma;
{
/*
 determine interval;
*/
      double tn, sigmap, del1, del2, dels, sumx, sumy, d, c1, c2, sigdel, ss;
      int im1, i;

      tn = s[n]*t;
      im1 = intrvl(tn,s,n);
      i = im1+1;
/*
 denormalize tension factor;
*/
      sigmap = fabs(sigma)*((double)(n-1))/s[n];
/*
 set up and perform interpolation;
*/
      del1 = tn-s[im1];
      del2 = s[i]-tn;
      dels = s[i]-s[im1];
      sumx = (x[i]*del1+x[im1]*del2)/dels;
      sumy = (y[i]*del1+y[im1]*del2)/dels;
      if (sigmap != 0.0) goto L1;
      d = del1*del2/(6.0*dels);
      c1 = (del1+dels)*d;
      c2 = (del2+dels)*d;
      *xs = sumx-xp[i]*c1-xp[im1]*c2;
      *ys = sumy-yp[i]*c1-yp[im1]*c2;
      return;
  L1: sigdel = sigmap*dels;
      ss = sinhmd(sigdel);
      d = sigdel*sigmap*(1.0+ss);
      c1 = del1*(sinhmd(sigmap*del1)-ss)/d;
      c2 = del2*(sinhmd(sigmap*del2)-ss)/d;
      *xs = sumx+xp[i]*c1+xp[im1]*c2;
      *ys = sumy+yp[i]*c1+yp[im1]*c2;
      return;
}

intrvl (t,x,n)
int n;
double t,x[];
{
      double tt;
      int il, ih, i;

      tt = t;
      if (tt < x[2]) return(1);
      if (tt >= x[n-1]) return(n-1);
      il = 2;
      ih = n-1;
/*
 linear interpolation;
*/
  L1: if (ih == il) return(il);
      i = il + (int) ((ih-il)*(tt-x[il])/(x[ih]-x[il]));
      if (i < 1) i = il;
      i = (i < ih-1) ? i : ih-1;
      if (tt >= x[i] && tt < x[i+1]) return(i);
      if (tt < x[i]) ih = i;
      else il = i+1;
      goto L1;
}

ceez (del1,del2,sigma,c1,c2,c3,n)
double del1,del2,sigma,*c1,*c2,*c3;
int n;
{
      double del, dummy, coshm1, coshm2, sinhmp, sinhmm, delp, delm, denom;

      if (n == 2) {
      *c1 = -1.0/del1;
      *c2 = -*c1;
      return;
      }
      if (sigma != 0.0) {
      snhcsh (&dummy,&coshm1,sigma*del1,1);
      snhcsh (&dummy,&coshm2,sigma*del2,1);
      delp = sigma*(del2+del1)/2.0;
      delm = sigma*(del2-del1)/2.0;
      snhcsh (&sinhmp,&dummy,delp,-1);
      snhcsh (&sinhmm,&dummy,delm,-1);
      denom = coshm1*(del2-del1)-2.0*del1*(sinhmp+delp)*(sinhmm+delm);
      *c1 = 2.0*(sinhmp+delp)*(sinhmm+delm)/denom;
      *c2 = -coshm2/denom;
      *c3 = coshm1/denom;
      return;
      }
      del = del2-del1;
      *c1 = -(del1+del2)/(del1*del2);
      *c2 = del2/(del1*del);
      *c3 = -del1/(del2*del);
      return;
}
 
terms (diag,sdiag,sigma,del)
double *diag,*sdiag,sigma,del;
{
      double sigdel,sinhm,coshm,denom;

      if (sigma != 0.0) goto L1;
      *diag = del/3.0;
      *sdiag = del/6.0;
      return;
  L1: sigdel = sigma*del;
      snhcsh (&sinhm, &coshm, sigdel, 0);
      denom = del/((sinhm+sigdel)*sigdel*sigdel);
      *diag = denom*(sigdel*coshm-sinhm);
      *sdiag = denom*sinhm;
      return;
}
