/*
 *	NOV. 1984 by Ching-Hua Chow
 *
 *	INTERACTIVE REACHABILITY ANALYSIS TOOL FOR PROTOCOL DESIGN
 * 
 *  Aug. 23, 1992 ported to xview by Ching-Hua Chow
 */

char designer[] = "Designed by Ching-Hua Chow and Simon S. Lam; Created by Ching-Hua Chow";

#include <stdio.h>

#include <xview/xview.h>
#include <xview/canvas.h>
#include <sys/stat.h>
#include <errno.h>
#include <xview/wmgr.h>
#include <xview/panel.h>
#include <xview/cursor.h>
#include <xview/svrimage.h>
#include <xview/font.h>
#include <xview/cms.h>
#include <xview/icon.h>
#include <xview/window.h>
#include <xview/notice.h>
#include <X11/Xlib.h>
#include <X11/X.h>
#include <fcntl.h>
#include "extxv.h"

#include "icon.h"

char font_name [] = "9x15";
char font6_name [] = "6x10";
Xv_Font font;
Xv_Font font6;

extern int	 errno;

#include "tool.h"

static u_int	label_cursor_array[8] = {
		0xFFFF8000, 0x80008000, 0x80008000, 0x80008000,
		0x80008000, 0x80008000, 0x80008000, 0x8000FFFF,
		};
static u_int	node_cursor_array[8] =  {
		0x01000100, 0x01000100, 0x01000100, 0x0100FFFE,
		0x01000100, 0x01000100, 0x01000100, 0x01000000, };
static u_int	link_cursor_array[8] = {
		0x0FE01010, 0x20084004, 0x80028002, 0x80028002,
		0x80028002, 0x80024004, 0x20081010, 0x0FE00000, };
static u_int	paint_cursor_array[8]  =  {
		0xC000E000, 0xF000F800, 0xFC00F000, 0x90001800,
		0x18000C00, 0x0C000600, 0x06000300, 0x03000100 };
static u_int	wait_cursor_array[8] = {
		0xFFE04040, 0x40403F80, 0x3F801F00, 0x0E000400,
		0x0A001100, 0x20802080, 0x40404040, 0xFFE00000, };


Server_image label_image;
Xv_Cursor	label_cursor;
Server_image node_image;
Xv_Cursor	node_cursor;
Server_image link_image;
Xv_Cursor	link_cursor;
Server_image paint_image;
Xv_Cursor	paint_cursor;
Server_image wait_image;
Xv_Cursor	wait_cursor;
Xv_Cursor	cur_cursor;

/*****************************   general tool area   **************************/

static char		 tool_name[]  =  "setool 1.1: STATE EXPLORATION TOOL FOR  PROTOCOL DESIGN, createdy by Ching-Hua Chow";
static int		 state;
#define			 WAIT 		1



Frame frame;
Xv_singlecolor fg, bg;

/****************************   popup subframe area  *********************/
Frame subframe;
Panel	popupPanel;
Panel_item	popupMsg_item;
Panel_item	popupData_item;
Panel_item	popupData2_item;

/****************************   popup protocolFrame area  *********************/
Frame protocolFrame;
Panel	protPanel;
Panel_item	protName_item;
Panel_item	machine1Name_item;
Panel_item	machine2Name_item;
/**************************** panel subwindow **********************/

Panel			 panel;
Panel_item msg_item; /* replace the old msw */
Panel_item txtmsg_item; /* replace the old msw */
Panel_item hightxtmsg_item; /* replace the old msw */
Panel_item highertxtmsg_item; /* replace the old msw */

#define PANEL_GC_KEY	101
/* labels for items in the order they occur */

int 			zoom = 1;
int 			oldzoom = 1;
Panel_item                 zoom_item;
void zoom_proc();

int                     arrow_head_drawing_mode = SOLID;
int                     save_arrow_head_drawing_mode;

Panel_item                 arrow_head_drawing_mode_item;
void    set_arrow_head_drawing_mode();

int                     link_drawing_mode = FAST;
int                     save_link_drawing_mode;
   
Panel_item                 link_drawing_mode_item;
void set_link_drawing_mode();

Panel_item                 comment_item;
void comment_proc();
void EDIT_MACHINE_proc();

/************************    painting area    ***************************/

Canvas canvas;
void create_template();

static struct pixrect	*fill_pr;
void	 	       (*canvas_reader)();
void		       position_reader();
			 detail_state_reader();
			 name_reader();
			 number_reader();
void ENTER_COMMENT_proc();
void CHANGE_COMMENT_proc();
void REMOVE_COMMENT_proc();
void			 canvas_selected();
void			 canvas_basereader();
void			 set_canvas_tracker();
void			 put_text();
char 			 save_text[BUF_LENGTH] = "";

/***********************    general globals    *****************************/

#include "menu.h"
int   (*menu_selected) ();
int     menu1_selected ();
int     menu2_selected ();
int     menu3_selected ();
int     menu5_selected ();
int     edit_menu_selected ();


FILE	       *sysout = stderr;

#include "g.h"
#include "state.h"

char graph_name[] = "M";
int			 cur_tp = 0;
int			 num_tp = 0;
double 			 hpi;
int  			  node_no = 0;
int  			  link_no = 0;
int  			  rspt_no = 0;
int  			 label_no = 0;
struct g_node		*first_node = 0;
struct g_node		*initial_node = 0;
struct g_node		*node;
struct g_node 		*node_highlighted = 0;

int			 link_type;
struct g_link		*first_link = 0;
struct g_link		*last_link = 0;
struct g_link		*cur_link = 0;
struct g_link		*link = 0;
struct g_link		*link_highlighted = 0;

struct g_rspt		*first_rspt = 0;
struct g_rspt		*rspt = 0;

struct g_label		*first_label = 0;
struct g_label		*label;
struct NODE_LIST *node_list = 0;
struct LINK_LIST *link_list = 0;
struct RSPT_LIST *rspt_list = 0;
struct LABEL_LIST *label_list  = 0;
char		 	 labelname[LABELNAME_LENGTH] = "";
short		 	 labelptr;
int		 	 cur_x, cur_y, fix_x, fix_y;

struct GRAPH		*g;
struct GRAPH		*g1;

int			 DX = 0;
int			 DY = 0;
int			 oldDX = 0;
int			 oldDY = 0;
short			 ERASE_GRAPH = 0;
int 	BB = 10; /* size of bounded buffer */
int     WS = 10;  /* size of flow control window */ 
int debug = 0;

int 			 x, y, X, Y;
int			 cswfd;
int			 mode = 9999;  /* current mode */

	                 node_tracker();
	                 link_tracker();
	                 common_tracker();
	                 movegraph_tracker();
	                 movenode_tracker();
			 movelink_tracker();

		 	 name_reader();
struct g_label	*label_search();
struct g_node    *node_search();

char		 buf[BUF_LENGTH];
char		 file_name[FILENAME_LENGTH] = "";  
char		 protocol_name[PROTOCOL_NAME_LENGTH] = "temp";
char		 process_name[MACHINE_NAME_LENGTH]= "M1";
char		 askname[ASKNAME_LENGTH];
char		 name[NAME_LENGTH];
short		 nameptr;
char 		 no[20];
short		 noptr;
int		 number;
short		 sw;
int               channel_type = UNI_DIRECTIONAL;
short		 namelength;

struct 	 PROCESS	P[2]; /* process names */
int		machine_exist[2] = { 0, 0};
struct STATE	ss[MAXSSNO];
int		cssno;
struct ARC	arc[MAXARCNO];
int		arc_no = -1;

struct STATELST	*outstatelst;
struct ARCLST	*outarclst;
/*********************************************************************/


short	rgno = 0; /* RG graph current max. no. */
		     
int 	cur_rgno = 1;
struct Rg	*rg[2];
int 		 rg_exist = 0;

struct PATH	path[PATH_NO];
int               cur_pathno;
struct MSGTBL	 msgtbl[MSG_TABLE_ENTRY_NO];
char   abnuse[SHORT_NAME_TBL_ENTRY_NO];
int    msgno = 0;
#define         BIG             2048

#define NUM_COLORS	8

void
select_machine_menu_proc(item, value, event)
Panel_item item;
int value;
Event *event;
{
	reset_canvas_reader();
}

void
modified(){
	g->status = MODIFIED;
}

void
protCancel_proc(item, event)
Panel_item item;
Event *event;
{
	xv_set(protocolFrame, XV_SHOW, FALSE, 0);
}

void
protData_proc(item, event)
Panel_item item;
Event *event;
{
	FILE *fd;
	char buf1[200];
	int i;

	xv_set(protocolFrame, XV_SHOW, FALSE, 0);
	strcpy(protocol_name, xv_get(protName_item, PANEL_VALUE));
	strcpy(P[0].name, xv_get(machine1Name_item, PANEL_VALUE));
	strcpy(P[1].name, xv_get(machine2Name_item, PANEL_VALUE));
	machine_exist[0] = 0;
	machine_exist[1] = 0;
	/* save two machine full names to a file */
	xv_set(msg_item, PANEL_LABEL_STRING,
		"Saving the machines of the protocol", 0);
	sprintf(file_name, "%s.process.names", protocol_name);
	if (file_name_ok(file_name) == 0) {
		reset_canvas_reader();
		return;
	}
	fd = fopen(file_name, "w");
	fprintf(fd, "%s\n",P[1].name);
	fprintf(fd, "%s\n",P[0].name);
	printf("M1 = %s, M2 = %s\n", P[1].name, P[0].name);
	fclose(fd);

	/* allocate space for the process graphs */

	for (i = 0; i < 2; i++) {
	    sprintf(file_name, "%s", P[i].name);
	    fd = fopen(file_name, "r");
	    if (fd == 0) {
			if (i==0) {
				sprintf(buf1, "Machine2 %s not exist, use machine menu to specify it, or specify another machine name.", P[0].name);
				put_text_high(buf1);
			} else {
				sprintf(buf1, "Machine1 %s not exist, use machine menu to specify it, or specify another machine name.", P[1].name);
				put_text_higher(buf1);
			}
			continue;
		}
	    load_graph_from_file(P[i].g, fd);
		machine_exist[i] = 1;
	    fclose(fd);
	    }
	paint_canvas();
	display_protocol();
}

void
popupCancel_proc(item, event)
Panel_item item;
Event *event;
{
	xv_set(popupData2_item, XV_SHOW, FALSE, 0);
	xv_set(subframe, XV_SHOW, FALSE, 0);
}

void
popupData_proc(item, event)
Panel_item item;
Event *event;
{
	int i;
	char data[300];
	char data2[300];

	xv_set(subframe, XV_SHOW, FALSE, 0);
	strcpy(data, xv_get(popupData_item, PANEL_VALUE));
	fprintf(stderr, "popupData=%s\n", data);
	switch(sw) {
	case INIT:
			number = atoi(data);
		  	paint_canvas(); 
			exploreRG0();
			sw = EXPL;
			return;
    case EXPL:
			number = atoi(data);
			mode = PLACERG;
			xv_set(msg_item, PANEL_LABEL_STRING, 
			  "please point the place where the root of the RG should be.", 0);
			return;
			
	case READING_PROTOCOL_NAME:
			strcpy(protocol_name, data);
			strcpy(askname, "Enter the machine name : ");
			strcpy(name, process_name);
			sw = READING_MACHINE_NAME;
			xv_set(popupData_item, PANEL_LABEL_STRING, askname,
		        PANEL_VALUE, name,
       			0);
			xv_set(subframe, XV_SHOW, TRUE, 0);
			return;
	case READING_MACHINE_NAME:
			strcpy(process_name, data);
			sprintf(file_name, "%s.%s", protocol_name, process_name);
			if (mode == LOAD) {
				release_objects();
				sprintf(buf, "Loading graph from %s", file_name);
				put_text_high(buf);
				load_file(file_name);
				
			} else if (mode == STORE) {
				if (file_name_ok(file_name) == 0) {
					reset_canvas_reader();
					return;
					}
				sprintf(buf, "Storing graph to %s", file_name);
				put_text_high(buf);
				store_file();
			} else if (mode == SAVE_SUB_GRAPH) {
				if (file_name_ok(file_name) == 0) {
					put_text("Please select the command in the middle button menu.");
					reset_canvas_reader();
					mode = 999;
					return;
					}
				put_text("Please point to the upper left conner of the sub-graph to be saved.");
				reset_canvas_reader();
				return;
				
			} else if (mode == CREATE_GRAPH) {
				if (file_name_ok(file_name) == 0) {
					reset_canvas_reader();
					return;
					}
			} else if (mode == PLACE_SUB_GRAPH) {
				canvas_reader = position_reader;
				put_text_high("Point to the upper left corner of the graph");
				put_text("  hit middle button for the upper left corner of the screen.");
				return;
			} else if (mode == EDIT_MACHINE) {
				strcpy(g->name, data);
				modified();
				if (g == P[0].g) strcpy(P[0].name, data);
				if (g == P[1].g) strcpy(P[1].name, data);
				draw_machine_name();
				}
			return;
	case READING_NODE_LABEL :
			erase_node_label(node);
			free(node->label);
			i = strlen(data) + 1;
			node->label = node->label = (char *) malloc(i*sizeof(char));
			strcpy(node->label, data);
			write_node_label(node);
			reset_canvas_reader();
			put_text("Please point to the node whose label needs to be changed.");
			return;
	case LOAD_GRAPH:
		strcpy(data2, xv_get(popupData2_item, PANEL_VALUE));
		fprintf(stderr, "popupData2=%s\n", data2);
		xv_set(popupData2_item, XV_SHOW, FALSE, 0);
		strcpy(process_name, data2);
		sprintf(file_name, "%s.%s", data, data2);
		release_objects();
		g->type = PG;
		sprintf(buf, "Loading machine from %s", file_name);
		put_text_higher(buf);
		load_file(file_name);
		break;
		
	case CREATE_GRAPH:
		strcpy(data2, xv_get(popupData2_item, PANEL_VALUE));
		fprintf(stderr, "popupData2=%s\n", data2);
		xv_set(popupData2_item, XV_SHOW, FALSE, 0);
		strcpy(protocol_name, data);
		strcpy(process_name, data2);
		sprintf(file_name, "%s.%s", data, data2);
		release_objects();
		g->type = PG;
		g->DX = DX = 0;
		g->DY = DY = 0;
		sprintf(buf, "Creating machine %s", file_name);
		put_text_higher(buf);
		put_text("Please select the editing commands in the Machine menu.");
		break;
		
	case STORE_GRAPH:
		strcpy(data2, xv_get(popupData2_item, PANEL_VALUE));
		fprintf(stderr, "popupData2=%s\n", data2);
		xv_set(popupData2_item, XV_SHOW, FALSE, 0);
		strcpy(process_name, data2);
		strcpy(protocol_name, data);
		sprintf(file_name, "%s.%s", data, data2);
		sprintf(buf, "Saving machine to File %s", file_name);
		put_text_higher(buf);
		if (file_name_ok(file_name) == 0) {
					reset_canvas_reader();
					return;
			}
		printf("g->node_no=%d\n", g->node_no);
		store_file();
		sprintf(buf, "Machine %s is saved", file_name);
		put_text_higher(buf);
		break;
		
	case REACHABILITY_ANALYSIS :
			if (mode == LOAD) {
				strcpy(protocol_name, data);
				load_protocol();
			}
			if (mode == STORE) {
				sprintf(buf, "%s.process.names", data);
				if (file_name_ok(buf) == 0) {
					reset_canvas_reader();
					return;
					}
				write_process_names(buf);
				write_machines(data);
				}
			if (mode == LOAD_RG) {
				strcpy(name, data);
				strcat(name, ".rg");
				sprintf(buf, "Loading reachability graph to %s", name);
				put_text_high(buf);
				load_rg_from_file(name);	
				}
			if (mode == STORE_RG) {
				strcpy(name, data);
				strcat(name, ".rg");
				if (file_name_ok(name) == 0) {
					reset_canvas_reader();
					return;
					}
				sprintf(buf, "Storing reachability graph to %s", name);	
				put_text_high(buf);
				store_rg_to_file(name);
				}
			reset_canvas_reader();
			put_text_high("");
			return;
		}
}

void
print_proc(item, event)
Panel_item item;
Event *event;
{
	system("/usr/openwin/bin/xwd | /usr/openwin/bin/xpr -device ps > temp.ps ");
}

void
exit_proc(item, event)
Panel_item item;
Event *event;
{
	int result;

	result = notice_prompt(panel, event,
				NOTICE_MESSAGE_STRINGS, "Are you sure you want to quit?",
					"Make sure you have saved all the needed result.", 0,
                NOTICE_BUTTON_YES, "Confirm",
                NOTICE_BUTTON_NO, "Cancel",
                0);			
	if (result == NOTICE_YES) exit(0);
}

initialize_cms_data(cms_data, red, green, blue)
Xv_cmsdata	*cms_data;
unsigned char *red, *green, *blue;
{
	/* white	*/ (red)[0] = 255; 	(green)[0] = 255; 	(blue)[0] = 255;
	/* red  	*/ (red)[1] = 255; 	(green)[1] = 0;		(blue)[1] = 0;
	/* green  	*/ (red)[2] = 0; 	(green)[2] = 255; 	(blue)[2] = 0;
	/* blue  	*/ (red)[3] = 0; 	(green)[3] = 0; 	(blue)[3] = 255;
	/* yellow  	*/ (red)[4] = 255; 	(green)[4] = 255; 	(blue)[4] = 0;
	/* brown  	*/ (red)[5] = 188; 	(green)[5] = 143; 	(blue)[5] = 143;
	/* gray  	*/ (red)[6] = 220; 	(green)[6] = 220; 	(blue)[6] = 220;
	/* black  	*/ (red)[7] = 0; 	(green)[7] = 0; 	(blue)[7] = 0;

	cms_data->type = XV_STATIC_CMS;
	cms_data->size = NUM_COLORS;
	cms_data->rgb_count = NUM_COLORS;
	cms_data->index = 0;
	cms_data->red = red;
	cms_data->green = green;
	cms_data->blue = blue;
}

paint_canvas()
{
	int w, h;

	w = (int) xv_get(canvas_paint_window(canvas), XV_WIDTH);
	h = (int) xv_get(canvas_paint_window(canvas), XV_HEIGHT);
	pwClearBackground(0, 0, w, h);
	rg_exist = 0;
	machine_exist[0] = 0;
	machine_exist[1] = 0;
}

init_g(g)
struct GRAPH *g;
{
	if (g == 0) return;
	strcpy(g->name, graph_name);
	g->node_no = 0;
	g->link_no = 0;
	g->rspt_no = 0;
	g->label_no = 0;
	g->first_node = 0;
	g->initial_node = 0;
	g->first_link = 0;
	g->last_link = 0;
	g->first_rspt = 0;
	g->first_label = 0;
	g->status = INITIALIZED;
}

display_protocol()
{
	struct GRAPH *save_g;

	save_g = g;
	put_channels();
	DX = -6; DY = 5;
	g = P[1].g;
	g->DX = DX; g->DY = DY;
	strcpy(g->name, P[1].name);
	if (g == 0) printf("P[1].g == NULL\n");
	draw_graph();
	DX = 500;
	g = P[0].g;
	g->DX = DX; g->DY = DY;
	strcpy(g->name, P[0].name);
	if (g == 0) printf("P[0].g == NULL\n");
	draw_graph();
	g = save_g;
	if (g == 0) {
		DX = 0; DY = 0;
		return;
		}
	DX = g->DX; DY = g->DY;
}

load_protocol()
{
	FILE	*fd;
	int i;

	put_text("");
	/* read in the process graphs */
	xv_set(msg_item, PANEL_LABEL_STRING,
		"Reading in the machines of the protocol", 0);
	sprintf(file_name, "%s.process.names", protocol_name);
	printf("%s\n", file_name);
	fd = fopen(file_name, "r");
	if (fd == 0) {
		put_text( "File not found!");
		return;
		}
	fscanf(fd, "%s\n",P[1].name);
	fscanf(fd, "%s\n",P[0].name);
	printf("P1 = %s, P2 = %s\n", P[1].name, P[0].name);
	fclose(fd);

	/* allocate space for the process graphs */

	for (i = 0; i < 2; i++) {
	    sprintf(file_name, "%s", P[i].name);
	    fd = fopen(file_name, "r");
	    if (fd == 0) return;
		printf("loading %s\n", P[i].name);
	    load_graph_from_file(P[i].g, fd);
	    fclose(fd);
	    }
	paint_canvas();
	display_protocol();
}

decide_abb_name_of_msg()
{
	int i;

	for ( i = 1; i > -1; i--) {
	   int j;
	   struct g_node *n;

	   for (n = P[i].g->first_node; n != 0; n = n->next) {
	      struct g_link	*a;
	      /* check all node's outarc labels */

	      for (a = n->outlink; a != 0; a = a->outlink) {
		 struct g_label *l;

		 l = a->label;
		 if ((l->name[0] != '+') && (l->name[0] != '-')) {
		    /* it is an internal transition. */
		    l->abn = l->name[0];
		    }
		 else l->abn = msg_abb_name_resolution(&(l->name[1]));
		 }
	      }
	   }
}

init_rg(rgno)
int rgno;
{	
	   g = rg[rgno]->g;
	   release_objects();
	   /* initialize state lists */
	   release_state_list(rg[rgno]->deadlock);
	   rg[rgno]->deadlock = 0;
	   release_state_list(rg[rgno]->unspecified_reception);
	   rg[rgno]->unspecified_reception = 0;
	   release_state_list(rg[rgno]->proper_termination);
   	   rg[rgno]->proper_termination = 0;
	   release_state_list(rg[rgno]->unexpstatelst);
	   rg[rgno]->unexpstatelst = 0;
}

exploreRG0()
{
	int i, maxlevel;

	maxlevel = number;
	/* decide the abb. name of the processes */
	for (i = 0; i < 79; i++) 
	    abnuse[i] = 0;
	msgno = 0;
	put_text( "Deciding the abb. name of the messages");
	decide_abb_name_of_msg();
	init_rg(0);
	init_rg(1);

	   paint_canvas();
	   print_msg_table();
	   put_text( "Exploring RG starting from initial state");
	   /* starting from initial state explore the RG */
	   for (i = 1; i > -1; i--) {
	      ss[0].pn[i] = P[i].g->initial_node->no;
	      ss[0].c[i].m[0] = (char) 0;
	      }
	   ss[0].property[0] = (char) 0;
	   ss[0].in = 0;
	   ss[0].out = 0;
	   ss[0].parentarcno = -1;
	   cssno = 0;
	   
	   /* do the reachability analysis */
	   g = rg[0]->g;
	   g->type = RG;
	   g->DX = 570; g->DY = 35;
	   cur_rgno = 0;
	   DX = 570; DY = 35;
	   explore(0, maxlevel, 0);
	   highlight_error_states(0);
	   /* print out the result of this exploration 
	      e.g. how many unexplored states, deadlock states? */
}

exploreRGn(ie)
struct inputevent       *ie;
{
	int i, ssn;
	struct Rglist *rl;


	paint_canvas();
	rgno = 1;
	init_rg(1);
	   
	   /* do the reachability analysis */
	   DX += ie->ie_locx;
	   DY += ie->ie_locy;
	   g->DX = DX;
	   g->DY = DY;
	   ssn = abs(node->ssno);
	   if(ssn == 99999) ssn = 0;
	   cur_rgno = rgno;
	   explore(ssn, number, rgno);
	   highlight_error_states(rgno);
	   /* print out the result of this exploration 
	      e.g. how many unexplored states, deadlock states? */
}

Xv_cmsdata	cms_data;

main(argc,argv)
int	  argc;
char	**argv;
{
	int i;
	extern char *optarg;
    extern int   optind;
	Icon	icon;
	int c;
	unsigned char	red[NUM_COLORS], green[NUM_COLORS], blue[NUM_COLORS];
	XGCValues	gcvalues;
	Server_image	gr1y;
	Mask	gcmask = 0L;
	GC	gc;

	xv_init(XV_INIT_ARGC_PTR_ARGV, &argc, argv, 0);
	initialize_cms_data(&cms_data, red, green, blue);
	wait_image = (Server_image)xv_create(0, SERVER_IMAGE,
		XV_WIDTH,	16,
		XV_HEIGHT,	16,
		SERVER_IMAGE_BITS,	wait_cursor_array,
		0);
	wait_cursor = (Xv_Cursor) xv_create(0, CURSOR,
		CURSOR_IMAGE,	wait_image,
		CURSOR_XHOT,	7,
		CURSOR_YHOT,	5,
		0);
	paint_image = (Server_image)xv_create(0, SERVER_IMAGE,
		XV_WIDTH,	16,
		XV_HEIGHT,	16,
		SERVER_IMAGE_BITS,	paint_cursor_array,
		0);
	paint_cursor = (Xv_Cursor) xv_create(0, CURSOR,
		CURSOR_IMAGE,	paint_image,
		CURSOR_XHOT,	0,
		CURSOR_YHOT,	0,
		0);
	link_image = (Server_image)xv_create(0, SERVER_IMAGE,
		XV_WIDTH,	16,
		XV_HEIGHT,	16,
		SERVER_IMAGE_BITS,	link_cursor_array,
		0);
	link_cursor = (Xv_Cursor) xv_create(0, CURSOR,
		CURSOR_IMAGE,	link_image,
		CURSOR_XHOT,	7,
		CURSOR_YHOT,	7,
		0);
	node_image = (Server_image)xv_create(0, SERVER_IMAGE,
		XV_WIDTH,	16,
		XV_HEIGHT,	16,
		SERVER_IMAGE_BITS,	node_cursor_array,
		0);
	node_cursor = (Xv_Cursor) xv_create(0, CURSOR,
		CURSOR_IMAGE,	node_image,
		CURSOR_XHOT,	7,
		CURSOR_YHOT,	7,
		0);
	label_image = (Server_image)xv_create(0, SERVER_IMAGE,
		XV_WIDTH,	16,
		XV_HEIGHT,	16,
		SERVER_IMAGE_BITS,	label_cursor_array,
		0);
	label_cursor = (Xv_Cursor) xv_create(0, CURSOR,
		CURSOR_IMAGE,	label_image,
		CURSOR_XHOT,	7,
		CURSOR_YHOT,	7,
		0);

	fg.red = 255; fg.green = 255; fg.blue=255;
	bg.red = 0; bg.green = 0; bg.blue = 0;
	frame = xv_create(0, FRAME,
		FRAME_INHERIT_COLORS, TRUE, 
		FRAME_FOREGROUND_COLOR, &fg,
		FRAME_BACKGROUND_COLOR, &bg,
		FRAME_LABEL,	  tool_name,
			 0);
	if (frame == 0) {
		fprintf(stderr, "Can't create frame");
		exit(1);
		}
	subframe = (Frame) xv_create(frame, FRAME_CMD,
		XV_WIDTH, 500,
		XV_HEIGHT, 100,
		FRAME_INHERIT_COLORS, TRUE,
		/*
		FRAME_FOREGROUND_COLOR, &fg,
		FRAME_BACKGROUND_COLOR, &bg,
		*/
		FRAME_LABEL, "Popup Command Frame",
		0);
	popupPanel = xv_create(subframe, PANEL, 
		/*
		WIN_CMS_NAME, "palette",
		WIN_CMS_DATA, &cms_data,
		WIN_FOREGROUND_COLOR, 7,
		WIN_BACKGROUND_COLOR, 4,
		*/
		XV_X, 10,
		XV_Y, 10,
		0);
	if (popupPanel == 0) exit(1);
	popupMsg_item = xv_create(popupPanel, PANEL_MESSAGE,
			PANEL_LABEL_STRING, "",
			PANEL_PAINT, PANEL_NO_CLEAR,
			XV_X, 10,
			XV_Y, 1,
			0);

	popupData_item = xv_create(popupPanel, PANEL_TEXT,
		PANEL_LABEL_STRING, "File:",
		PANEL_NOTIFY_LEVEL, PANEL_NONE,
		PANEL_NOTIFY_PROC, popupData_proc,
		XV_X, 10,
		XV_Y, 17,
		0);
	popupData2_item = xv_create(popupPanel, PANEL_TEXT,
		PANEL_LABEL_STRING, "",
		PANEL_NOTIFY_LEVEL, PANEL_NONE,
		XV_X, 10,
		XV_Y, 34,
		XV_SHOW, FALSE,
		0);
	xv_create(popupPanel, PANEL_BUTTON,
		PANEL_LABEL_STRING, "DO IT!",
		PANEL_NOTIFY_PROC, popupData_proc,
		XV_X, 10,
		XV_Y, 51,
		0);
	xv_create(popupPanel, PANEL_BUTTON,
		PANEL_LABEL_STRING, "CANCEL!",
		PANEL_NOTIFY_PROC, popupCancel_proc,
		XV_X, 100,
		XV_Y, 51,
		0);
	window_fit(popupPanel);
	window_fit(subframe);
	protocolFrame = (Frame) xv_create(frame, FRAME_CMD,
		XV_WIDTH, 500,
		XV_HEIGHT, 100,
		FRAME_INHERIT_COLORS, TRUE,
		/*
		FRAME_FOREGROUND_COLOR, &fg,
		FRAME_BACKGROUND_COLOR, &bg,
		*/
		FRAME_LABEL, "Create Protocol Frame",
		0);
	protPanel = xv_create(protocolFrame, PANEL, 
	/*
		WIN_CMS_NAME, "palette",
		WIN_CMS_DATA, &cms_data,
		WIN_FOREGROUND_COLOR, 7,
		WIN_BACKGROUND_COLOR, 4,
		*/
		XV_X, 10,
		XV_Y, 10,
		0);
	if (protPanel == 0) exit(1);
	protName_item = xv_create(protPanel, PANEL_TEXT,
		PANEL_LABEL_STRING, "Please enter new protocol name:",
		PANEL_NOTIFY_LEVEL, PANEL_NONE,
		PANEL_VALUE, "temp",
		XV_X, 10,
		XV_Y, 1,
		0);

	machine1Name_item = xv_create(protPanel, PANEL_TEXT,
		PANEL_LABEL_STRING, "Please enter machine 1's full name:",
		PANEL_NOTIFY_LEVEL, PANEL_NONE,
		PANEL_VALUE, "temp.S",
		XV_X, 10,
		XV_Y, 17,
		0);
	machine2Name_item = xv_create(protPanel, PANEL_TEXT,
		PANEL_LABEL_STRING, "Please enter machine 2's full name:",
		PANEL_NOTIFY_LEVEL, PANEL_NONE,
		PANEL_VALUE, "temp.R",
		XV_X, 10,
		XV_Y, 34,
		0);
	xv_create(protPanel, PANEL_BUTTON,
		PANEL_LABEL_STRING, "DO IT!",
		PANEL_NOTIFY_PROC, protData_proc,
		XV_X, 10,
		XV_Y, 51,
		0);
	xv_create(protPanel, PANEL_BUTTON,
		PANEL_LABEL_STRING, "CANCEL!",
		PANEL_NOTIFY_PROC, protCancel_proc,
		XV_X, 100,
		XV_Y, 51,
		0);
	window_fit(protPanel);
	window_fit(protocolFrame);
	/*
	icon = (Icon) xv_create(0, ICON, 
		ICON_IMAGE,	myicon,
		0);
	*/
	icon = xv_get(frame, FRAME_ICON);
	xv_set(icon, ICON_LABEL, "PDS", 0);
	xv_set(frame, FRAME_ICON, icon, 0);
	font = xv_find(frame, FONT, FONT_NAME, font_name, 0);
	if (!font) {
		fprintf(stderr, "%s: Cannot use font: %s.\n", argv[0], font_name);
		font = (Xv_Font) xv_get(frame, XV_FONT);
	}
	font6 = xv_find(frame, FONT, FONT_NAME, font6_name, 0);
	if (!font) {
		fprintf(stderr, "%s: Cannot use font: %s.\n", argv[0], font6_name);
		font6 = (Xv_Font) xv_get(frame, XV_FONT);
	}
	fprintf(stderr, "after create font.\n" );
	fprintf(stderr, "after create GC.\n" );
	init_menu();
	panel = xv_create(frame, PANEL, 
		XV_WIDTH, 1000,
		XV_HEIGHT, 55,
		/*
		WIN_CMS_NAME, "palette",
		WIN_CMS_DATA, &cms_data,
		WIN_FOREGROUND_COLOR, 7,
		WIN_BACKGROUND_COLOR, 5,
		*/
		0);
	if (panel == 0) exit(1);
	init_panel();
	fprintf(stderr, "after init_panel.\n" );

	window_fit_height(panel);
	fprintf(stderr, "after init_menu.\n" );
	/* initial value */

	hpi = asin(1.0);
	g = (struct GRAPH *) malloc(sizeof(struct GRAPH));
	init_g(g);
	g->type = PG;
	g1 = (struct GRAPH *) malloc(sizeof(struct GRAPH));
	g1->type = PG;
	init_g(g1);
	for (i = 0; i < 2; i++) {
	    if (i == 0) strcpy(P[i].name, "M2");
	    else strcpy(P[i].name, "M1");
	    P[i].g = (struct GRAPH *) malloc(GRAPH_SIZE);
	    init_g(P[i].g);
		strcpy(P[i].g->name, P[i].name);
	    P[i].g->type = PG;
	    P[i].g->DX = 0;
	    P[i].g->DY = 0;
	    P[i].g->no = (i == 1) ? 1 : 2;
	    }
	   /* initialize RG[0] */
	   rg[0] = (struct Rg *) malloc(Rg_SIZE);
	   rg[0]->g  = (struct GRAPH *) malloc(GRAPH_SIZE);
	   rg[0]->g->no = 0;
	   init_g(rg[0]->g);
	   rg[0]->g->type = RG;
	   rg[0]->g->DX = 570; 
	   rg[0]->g->DY = 35; 
	   rg[0]->parentrgno = 0;
	   rg[0]->child = 0;
	   /* initialize state lists,  may not be necessary*/
	   rg[0]->deadlock = 0;
	   rg[0]->unspecified_reception = 0;
	   rg[0]->proper_termination = 0;
	   rg[0]->unexpstatelst = 0;

	   /* initialize RG[1] */
	   rg[1] = (struct Rg *) malloc(Rg_SIZE);
	   rg[1]->g = (struct GRAPH *) malloc(GRAPH_SIZE);
	   rg[1]->g->no = 1;
	   init_g(rg[1]->g);
	   rg[1]->g->type = RG;
	   rg[1]->parentrgno = 0;
	   rg[1]->child = 0;
	   /* initialize state lists */
	   rg[1]->deadlock = 0;
	   rg[1]->unspecified_reception = 0;
   	   rg[1]->proper_termination = 0;
	   rg[1]->unexpstatelst = 0;


    /* process command line args */
    while ((c=getopt(argc,argv,"db:w:")) != EOF)
    {
        switch(c)
        {
        case 'd':
            debug = 1;
            break;

        case 'b':
            if ((BB=atoi(optarg)) <= 1) BB=10 ;
            break;

        case 'w':
            if ((WS=atoi(optarg)) <= 1) WS=10 ;
            break;

		case '?':
            printf("command syntax: setool -b <size of bouned FIFO>\n");
			printf("                       -d /* turn on debuggin mode */\n");
			printf("                       -w <size of flow control window>\n");
			printf("                       protocol_name \n");

        default:
			fprintf (stderr, "unrecognized arg >%s<\n", optarg);
            exit (1);
        }
	}

	if (BB < WS) {
		printf("window size should not be bigger than buffer\n");
		exit(1);
		}

	if (argv[optind] != 0) {
		strcpy(protocol_name, argv[optind]);
		if (debug) printf("protocol name = %s\n", protocol_name);
		load_protocol();
		mode = MACHINE_EXIST;/* set mode to DETAIL so that we can edit machine
		of the protocol without redisplaying them; see menu.c on case EDIT_MACHINE case  */
		}
	canvas = xv_create(frame, CANVAS,
		CANVAS_WIDTH, 1000,
		CANVAS_HEIGHT, 700,
		XV_X, 0,
		XV_Y, 90,
		WIN_FOREGROUND_COLOR, &fg,
		WIN_BACKGROUND_COLOR, &bg,
		/*
		WIN_CMS_NAME, "palette",
		WIN_CMS_DATA, &cms_data,
		*/
		0);
	if (canvas == 0) exit(1);
	init_canvas();
	window_fit(canvas);
	fprintf(stderr, "after init_canvas.\n" );
	window_fit(frame);
	init_GC();
	create_template();
	fprintf(stderr, "after init_GC()\n");
	paint_canvas();
	xv_main_loop(frame);
	exit(0);
	}

void 
setZoomMode() 
{
        mode = ZOOM;
        canvas_reader = position_reader;
        put_text("Please point to the position to be the center of zooming");
}

void
NORMAL_proc(item, event)
Panel_item item;
Event *event;
{
	zoom = 1;
	setZoomMode();
}

void
HALF_proc(item, event)
Panel_item item;
Event *event;
{
	zoom = 2;
	setZoomMode();
}

void
QUAR_proc(item, event)
Panel_item item;
Event *event;
{
	zoom = 4;
	setZoomMode();
}

void
EIGHTH_proc(item, event)
Panel_item item;
Event *event;
{
	zoom = 8;
	setZoomMode();
}

void
SIXTEENTH_proc(item, event)
Panel_item item;
Event *event;
{
	if (g->type != RG) {
          put_text("ZoomScale 16 Only valid for Reachability Graph!");
          return;
    }
	zoom = 16;
	setZoomMode();
}

init_panel()
{
	Menu commentMenu;
	Menu zoomScaleMenu;

	msg_item = xv_create(panel, PANEL_MESSAGE,
			PANEL_LABEL_STRING, designer,
			PANEL_PAINT, PANEL_NO_CLEAR,
			XV_X, 10,
			XV_Y, 1,
			/*
			WIN_FOREGROUND_COLOR, 7,
			WIN_BACKGROUND_COLOR, 2,
			*/
			0);

	txtmsg_item = xv_create(panel, PANEL_MESSAGE,
            PANEL_LABEL_STRING, "",
			XV_X, 10,
			XV_Y, 15,
			/*
			WIN_FOREGROUND_COLOR, 7,
			WIN_BACKGROUND_COLOR, 2,
			*/
            0);

	hightxtmsg_item = xv_create(panel, PANEL_MESSAGE,
            PANEL_LABEL_STRING, "",
			XV_X, 10,
			XV_Y, 30,
			/*
			WIN_FOREGROUND_COLOR, 7,
			WIN_BACKGROUND_COLOR, 2,
			*/
            0);

	highertxtmsg_item = xv_create(panel, PANEL_MESSAGE,
            PANEL_LABEL_STRING, "",
			XV_X, 10,
			XV_Y, 45,
            0);
	(void) xv_create(panel, PANEL_BUTTON,
			PANEL_LABEL_STRING, "Protocol",
			PANEL_ITEM_MENU, menu1,
			XV_X, 10,
			XV_Y, 60,
			/*
			WIN_FOREGROUND_COLOR, 7,
			WIN_BACKGROUND_COLOR, 4,
			*/
			0);
	(void) xv_create(panel, PANEL_BUTTON,
			PANEL_LABEL_STRING, "Machine",
			PANEL_ITEM_MENU, edit_menu,
		    PANEL_NOTIFY_PROC, select_machine_menu_proc,
			XV_X, 110,
			XV_Y, 60,
			/*
			WIN_FOREGROUND_COLOR, 7,
			WIN_BACKGROUND_COLOR, 4,
			*/
			0);
	zoomScaleMenu = xv_create(0, MENU,
		MENU_TITLE_ITEM, "Zoom Scale",
		MENU_ITEM, MENU_STRING, "1", 	
			MENU_NOTIFY_PROC, NORMAL_proc	, 0,
		MENU_ITEM, MENU_STRING, "1/2",	
			MENU_NOTIFY_PROC, HALF_proc	, 0,
		MENU_ITEM, MENU_STRING, "1/4",	
			MENU_NOTIFY_PROC, QUAR_proc	, 0,
		MENU_ITEM, MENU_STRING, "1/8", 	
			MENU_NOTIFY_PROC, EIGHTH_proc	, 0,
		MENU_ITEM, MENU_STRING, "1/16", 
			MENU_NOTIFY_PROC, SIXTEENTH_proc , 0,
		0);
	zoom_item = xv_create(panel, PANEL_BUTTON,
			PANEL_LABEL_STRING, "Zoom",
			PANEL_ITEM_MENU, zoomScaleMenu,
			XV_X, 200,
			XV_Y, 60,
			/*
			WIN_FOREGROUND_COLOR, 7,
			WIN_BACKGROUND_COLOR, 4,
			*/
			0);
    commentMenu = xv_create(0, MENU,
        MENU_TITLE_ITEM, "Comment",
        MENU_ITEM, MENU_STRING, "enter comment",
            MENU_NOTIFY_PROC, ENTER_COMMENT_proc    , 0,
        MENU_ITEM, MENU_STRING, "change comment",
            MENU_NOTIFY_PROC, CHANGE_COMMENT_proc   , 0,
        MENU_ITEM, MENU_STRING, "remove comment",
            MENU_NOTIFY_PROC, REMOVE_COMMENT_proc   , 0,
        0);

	comment_item = xv_create(panel, PANEL_BUTTON,
		 	PANEL_LABEL_STRING, "Comment",
			PANEL_ITEM_MENU, commentMenu,
			/*
			WIN_FOREGROUND_COLOR, 7,
			WIN_BACKGROUND_COLOR, 4,
			*/
			XV_X, 270,
			XV_Y, 60,
				      0);
	(void) xv_create(panel, PANEL_BUTTON,
			PANEL_LABEL_STRING, "Edge Type",
			PANEL_ITEM_MENU, menu2,
			XV_X, 365,
			XV_Y, 60,
			/*
			WIN_FOREGROUND_COLOR, 7,
			WIN_BACKGROUND_COLOR, 4,
			*/
			0);
	(void) xv_create(panel, PANEL_BUTTON,
			PANEL_LABEL_STRING, "Quit",
		    PANEL_NOTIFY_PROC, exit_proc,
			XV_X, 480,
			XV_Y, 60,
			/*
			WIN_FOREGROUND_COLOR, 7,
			WIN_BACKGROUND_COLOR, 4,
			*/
		    0);
	(void) xv_create(panel, PANEL_BUTTON,
			PANEL_LABEL_STRING, "Print",
		    PANEL_NOTIFY_PROC, print_proc,
			XV_X, 550,
			XV_Y, 60,
			/*
			WIN_FOREGROUND_COLOR, 7,
			WIN_BACKGROUND_COLOR, 4,
			*/
		    0);
	arrow_head_drawing_mode_item = 
		    xv_create(panel, PANEL_CHOICE,
			PANEL_ITEM_Y, 0,
			PANEL_LABEL_STRING, "Arrow Head",
			PANEL_CHOICE_STRINGS, "solid",
			"spear",
		    "no",
			0,
		    PANEL_NOTIFY_PROC, set_arrow_head_drawing_mode,
			XV_X, 610,
			XV_Y, 60,
			/*
			WIN_FOREGROUND_COLOR, 7,
			WIN_BACKGROUND_COLOR, 4,
			*/
				      0);
	link_drawing_mode_item = 
		    xv_create(panel, PANEL_CHOICE,
				      PANEL_ITEM_Y, 0,
				      PANEL_LABEL_STRING, "Draw",
				      PANEL_CHOICE_STRINGS, 
						"delicately", 
						"fast", 
						0,
					  PANEL_VALUE, FAST,
				      PANEL_NOTIFY_PROC, set_link_drawing_mode,
			XV_X, 800,
			XV_Y, 60,
			/*
			WIN_FOREGROUND_COLOR, 7,
			WIN_BACKGROUND_COLOR, 4,
			*/
				      0);
}

void
set_link_drawing_mode(item, value, event)
Panel_item item;
int value;
Event *event;
{
	link_drawing_mode = value;
	save_link_drawing_mode = value;
	paint_canvas();
	draw_graph();
}


void
set_arrow_head_drawing_mode(item, value, event)
Panel_item item;
int value;
Event *event;
{
	arrow_head_drawing_mode = value;
	save_arrow_head_drawing_mode = value;
	paint_canvas();
	draw_graph();
}


nullproc()
{
	return;
}


int   (*saved_handler)();
int	saved_mask;
int	clear_message();

warning(format, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8)
char   *format, *arg1, *arg2, *arg3, *arg4, *arg5, *arg6, *arg7, *arg8;
{
	char	buf[256];

	sprintf(buf,format, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8);
	put_text(buf);
	/*
	saved_mask = tool->tl_io.tio_inputmask;
	tool->tl_io.tio_inputmask = (1 << tool->tl_windowfd)  +
				    (1 << cswfd); 
	if (tool->tl_io.tio_selected != clear_message)  {
		saved_handler = tool->tl_io.tio_selected;
	}
	tool->tl_io.tio_selected = clear_message;
	*/
}

clear_message(datum, ibits, obits, ebits, timer)
caddr_t		 datum;
u_int		*ibits, *obits, *ebits;
struct timeval **timer;
{
	put_text("");
	/*
	tool->tl_io.tio_selected = saved_handler;
	tool->tl_io.tio_inputmask = saved_mask;
	*/
}

/*
 *	 Canvas Section
 */

init_canvas()
{
	menu_selected = menu1_selected;
	canvas_reader = canvas_basereader;

	xv_set(canvas_paint_window(canvas), 
		WIN_EVENT_PROC, canvas_selected,
		WIN_CONSUME_EVENTS, 
			WIN_ASCII_EVENTS, LOC_DRAG, LOC_MOVE, LOC_WINENTER, LOC_WINEXIT,
			WIN_MOUSE_BUTTONS, 0,
		WIN_CURSOR, paint_cursor, 
		0);
	
}

void
canvas_selected(window, event)
Xv_Window window;
Event *event;
{
	(*canvas_reader)(event);
}

void
canvas_basereader(event)
Event *event;
{

	fprintf(stderr,  "event=%d, x=%d, y=%d\n", event->ie_code, 
			event->ie_locx, event->ie_locy);
	switch (event_action(event)) {
	  case ACTION_SELECT:	  
		set_tracker(event);
		return;
/*
	  case ACTION_ADJUST: 
		(*menu_selected) (event);
		return;
*/
	  default:
		return;
	}
}

reset_canvas_reader()
{
	canvas_reader = canvas_basereader;
}

/*
winsetmouseposition(cswfd, x, y)
int cswfd, x, y;
{
	win_setmouseposition(cswfd, (x+DX)/zoom, (y+DY)/zoom);
}
*/

