#include <stdio.h>    /* this is where NULL is defined */
#include <xview/xview.h>
#include "tool.h"
#include "g.h"
#include "state.h"
#include "ext.h"

struct g_node *
node_search(x, y, res)
int	x, y, res;
{
	struct g_node	*p;
	int		 minx, maxx, miny, maxy;

	if (g == NULL) return(NULL);
	minx = x-res;
	maxx = x+res;
	miny = y-res;
	maxy = y+res;
	for (p = g->first_node; p != NULL; p = p->next) {
	    if (p->x < minx) continue;
	    if (p->x > maxx) return (NULL);
	    if (p->y < miny) continue;
	    if (p->y > maxy) continue;
	    return(p); 
	    }
	return(NULL);
	}

insert_node(node)
struct g_node	*node;
{
	register int	 x, y;
	struct g_node	*q, *r;

	modified();
	x = node->x;
	y = node->y;
	for (r = g->first_node; r != NULL; q = r, r = r->next) {
	    if (r->x > x) break;
	    if (r->x < x) continue;
	    if (r->y < y) continue;
	    if (r->y >= y) break;
	    }
	if (r == g->first_node) 
	    g->first_node = node;
	else
	    q->next = node;
	node->next = r;
	}

delete_node(n)
struct g_node	*n;
{
	struct g_node	*q, *r;

	if (g->first_node == NULL) return;
	if (n == NULL) return;

	for (r = g->first_node; r != NULL; q = r, r = r->next) {
	    if (r == n) {
		modified();
		if (r == g->first_node)
		    g->first_node = n->next;
		else
		    q->next = n->next;
		break;
		}
	    }
	}

free_node(n)
struct g_node *n;
{
	if (n->label != NULL) free(n->label);
	free(n);
}

void
init_nodetracking(ie)
Event	*ie;
{
	cur_x = ie->ie_locx; 
	cur_y = ie->ie_locy;
	draw_node(cur_x, cur_y);
	}

node_tracker(ie)
struct	inputevent  *ie;
{

	ie->ie_locx = zoom*ie->ie_locx - DX;
	ie->ie_locy = zoom*ie->ie_locy - DY;
	if (ie->ie_code == MS_LEFT)  {
	   place_node(ie);
	   reset_canvas_reader();
	   return;
	   }
	switch (ie->ie_code) {
          case LOC_WINEXIT:
	  case MS_MIDDLE :		 /*  two buttons down?	     */ 
		erase_node(cur_x, cur_y);
		reset_canvas_reader();
		return;
	  case LOC_MOVE:
		erase_node(cur_x, cur_y);
		cur_x = ie->ie_locx;   
		cur_y = ie->ie_locy;
		draw_node(cur_x, cur_y);
		return;
	  }				/* ignore all other input  */
	}

place_node(ie)
Event	*ie;
{

	if (cur_x != ie->ie_locx || cur_y != ie->ie_locy) {
	   erase_node(cur_x, cur_y);
	   cur_x = ie->ie_locx;
	   cur_y = ie->ie_locy;
	   draw_node(cur_x, cur_y);
	   }

	node = (struct g_node *) malloc(G_NODE_SIZE);
	node->no    = ++g->node_no;
	modified();
	node->x     = cur_x;
	node->y     = cur_y;
	node->next  = NULL;
	node->inlink = NULL;
	node->outlink = NULL;
	node_label(node); /* in pllabel.c */
	if (node->no == 1) {
		g->initial_node = node;
		draw_initial_node_symbol(node);
		}
	insert_node(node);
	}

static int dx, dy;

init_nodemoving(ie)
Event	*ie;
{

	if (g == NULL) return(1);
	node = node_search(ie->ie_locx, ie->ie_locy, 
		((g->type == PA) || (g->type == RG)) ? STATE_TOL : NODE_TOL);
	if (node == NULL) {
		return(1);
	}

	cur_x = node->x;
	cur_y = node->y;
	/* winsetmouseposition(cswfd, cur_x, cur_y); */
	return(0);

}

struct g_rspt *
last_restpt(rp)
struct g_rspt	*rp;
{
	struct g_rspt	*p;

	for (p = NULL; rp != NULL; p = rp, rp = rp->next_inlink) ;
	return(p);
}

movenode_tracker(ie)
Event	*ie;
{


	ie->ie_locx = zoom*ie->ie_locx - DX;
	ie->ie_locy = zoom*ie->ie_locy - DY;
	switch(ie->ie_code) {
	case LOC_MOVE:
		erase_nodelinks();
		dx = ie->ie_locx - cur_x;
		dy = ie->ie_locy - cur_y;
		cur_x = ie->ie_locx;
		cur_y = ie->ie_locy;
		draw_nodelinks();
		return;
	case LOC_WINEXIT :
		return;
	case CNTL_C :
		erase_nodelinks();
		dx = fix_x - cur_x;
		dy = fix_y - cur_y;
		cur_x = fix_x;
		cur_y = fix_y; /* store the old location back */
		draw_nodelinks();
	case MS_MIDDLE :
		relocate_node();
		reset_canvas_reader();
		replace_straight_links_with_links(node);
		put_text_high("");
		put_text("Please point to the node to be moved .");
		return;
	}
}

op_nodelinks(sw)
int sw;
{
	point cur, p, q, s;
	struct g_rspt *r;
	struct g_link *l;

	cur.x = cur_x;
	cur.y = cur_y;
	for (l = node->inlink; l != NULL ; l = l->inlink) {
		if (l->src_port.node == l->dst_port.node) {
			/* self loop */
			if (sw == ERASE_NODELINKS) {
				erase_link(l);
				erase_edge_label(l->label);
			} else {
			l->src_port.x += dx;
			l->src_port.y += dy;
			l->dst_port.x += dx;
			l->dst_port.y += dy;
			for (rspt = l->rest_points; rspt != NULL; rspt = rspt->next_inlink) {
				delete_rspt(rspt);
				rspt->x += dx;
				rspt->y += dy;
				insert_rspt(rspt);
				}
			if (l->label != NULL) {
				delete_label(l->label);
				l->label->x += dx;
				l->label->y += dy;
				insert_label(l->label);
				write_edge_label(l->label);
				}
				draw_link(l);
				}
			continue;
			}
		if (sw == ERASE_NODELINKS) { 
			if (l->rest_points != NULL) {
				r = last_restpt(l->rest_points);
				s.x = r->x;
				s.y = r->y;
			}
			else {
				s.x = l->src_port.x;
				s.y = l->src_port.y;
				erase_edge_label(l->label);
			}
			erase_arc(s.x, s.y, l->dst_port.x, l->dst_port.y);
		}
		if (sw == DRAW_NODELINKS)  { 
			if (l->rest_points != NULL) {
				r = last_restpt(l->rest_points);
				q.x = s.x = r->x;
				q.y = s.y = r->y;
			}
			else {
				s.x = l->src_port.node->x;
				s.y = l->src_port.node->y;
				find_port(cur, s, &q);
				l->src_port.x = q.x;
				l->src_port.y = q.y;
			}
			find_port(s, cur, &p);
			draw_arc(q.x, q.y, p.x, p.y);
			if (l->rest_points == NULL) {
				if (l->label != NULL) {
					l->label->x = (l->src_port.x + l->dst_port.x) / 2 + 3;
					l->label->y = (l->src_port.y + l->dst_port.y) / 2 - 3;
					write_edge_label(l->label);
					}
			}
			l->dst_port.x = p.x;
			l->dst_port.y = p.y;
		}
	}
	for (l = node->outlink; l != NULL ; l = l->outlink) {
		if (l->dst_port.node == l->src_port.node) continue;
		if (l->rest_points != NULL) {
			r = l->rest_points; /* first rest point in the link */
			s.x = r->x;
			s.y = r->y;
		}
		else {
			s.x = (l->dst_port.node)->x;
			s.y = (l->dst_port.node)->y;
		}
		find_port(s, cur, &p);
		if (sw == DRAW_NODELINKS)  { 
			if (l->rest_points == NULL) {
				find_port(cur, s, &q);
				draw_arc(p.x, p.y, q.x, q.y);
				l->dst_port.x = q.x;
				l->dst_port.y = q.y;
				if (l->label != NULL) {
					l->label->x = (l->src_port.x + l->dst_port.x) / 2 + 3;
					l->label->y = (l->src_port.y + l->dst_port.y) / 2 - 3;
					write_edge_label(l->label);
					}
			}
			else draw_vector(p.x, p.y, s.x, s.y);
			l->src_port.x = p.x;
			l->src_port.y = p.y;
		}
		if (sw == ERASE_NODELINKS) { 
			if (l->rest_points == NULL) {
				erase_edge_label(l->label);
				erase_arc(l->src_port.x, l->src_port.y, l->dst_port.x,
				l->dst_port.y);
			}
			else erase_vector(l->src_port.x, l->src_port.y, s.x, s.y);
		}
	} 
}

replace_links_with_straight_links(n)
struct g_node *n;
{
	for (link = n->inlink; link != NULL; link = link->inlink) {
		erase_link(link);
		erase_edge_label(link->label);
		}
	for (link = n->outlink; link != NULL; link = link->outlink) {
		if (link->dst_port.node == link->src_port.node) continue;
		erase_link(link);
		erase_edge_label(link->label);
		}
	save_link_drawing_mode = link_drawing_mode;
	save_arrow_head_drawing_mode = arrow_head_drawing_mode;
	link_drawing_mode = FAST;
	arrow_head_drawing_mode = NO_ARROW_HEAD;
	for (link = n->inlink; link != NULL; link = link->inlink) {
		write_edge_label(link->label);
		draw_link(link);
		}
	for (link = n->outlink; link != NULL; link = link->outlink) {
		if (link->dst_port.node == link->src_port.node) continue;
		write_edge_label(link->label);
		draw_link(link);
		}
}

replace_straight_links_with_links(n)
struct g_node *n;
{
	for (link = n->inlink; link != NULL; link = link->inlink) {
		erase_link(link);
		erase_edge_label(link->label);
		}
	for (link = n->outlink; link != NULL; link = link->outlink) {
		if (link->dst_port.node == link->src_port.node) continue;
		erase_link(link);
		erase_edge_label(link->label);
		}
	link_drawing_mode = save_link_drawing_mode;
	arrow_head_drawing_mode = save_arrow_head_drawing_mode;
	for (link = n->inlink; link != NULL; link = link->inlink) {
		draw_link(link);
		write_edge_label(link->label);
		}
	for (link = n->outlink; link != NULL; link = link->outlink) {
		if (link->dst_port.node == link->src_port.node) continue;
		draw_link(link);
		write_edge_label(link->label);
		}
}

erase_nodelinks()
{
	op_nodelinks(ERASE_NODELINKS);
	erase_node_label(node);
	if (node == g->initial_node) erase_initial_node_symbol(node); 
	erase_node(cur_x, cur_y);
}

draw_nodelinks()
{
	draw_node(cur_x, cur_y);
	node->x = cur_x;
	node->y = cur_y;
	write_node_label(node);
	op_nodelinks(DRAW_NODELINKS);
	if (node == g->initial_node) draw_initial_node_symbol(node); 
}

relocate_node()
{

	delete_node(node);
	node->x = cur_x;
	node->y = cur_y;
	node->next = NULL;
	insert_node(node);
	modified();

}

movegraph_tracker(ie)
Event	*ie;
{

	ie->ie_locx = zoom*ie->ie_locx - DX;
	ie->ie_locy = zoom*ie->ie_locy - DY;
	if (ie->ie_code == MS_MIDDLE || ie->ie_code == CNTL_C) {
		if (ie->ie_code == MS_MIDDLE) relocate_graph();
		else { DX = oldDX; DY = oldDY;}
		reset_canvas_reader();
		paint_canvas();
		link_drawing_mode = save_link_drawing_mode;
		arrow_head_drawing_mode = save_arrow_head_drawing_mode;
		draw_graph();
		clear_msg();
		put_text("Please point to the reference point.");
		return;
	}

	if (ie->ie_code == MS_LEFT) {
		erase_graph();
		DX += ie->ie_locx - cur_x;
		DY += ie->ie_locy - cur_y;
		X -= (ie->ie_locx - cur_x);
		Y -= (ie->ie_locy - cur_y);
		draw_graph();
		return;
	}
	if (ie->ie_code == LOC_MOVE) {
		erase_vector(fix_x + oldDX - DX, fix_y + oldDY - DY, X, Y);
		X = ie->ie_locx;
		Y = ie->ie_locy;
		draw_vector(fix_x + oldDX - DX, fix_y + oldDY - DY, X , Y );
		}


}

rm_link(link)
struct g_link 	*link;
{
	struct g_link *L, *prev, *b, *f;

	for (L = g->first_link, prev = NULL; L != NULL; prev = L, L = L->next) {
	    if (L == link) {
		modified();
		op_link(link, ERASE);
		remove_restpt(link);
		if (link == g->first_link)
			g->first_link = link->next;
		else    prev->next = link->next;
		if (link == g->last_link) g->last_link = prev; 
		remove_label(link);
		remove_link_from_inlink_and_outlink(link);
		free(link);
		return;
		}
	}
}

rm_node(n)
struct g_node *n;
{
	struct g_link *link;

	if (n == NULL) return;
	erase_node(n->x, n->y);
	erase_node_label(n);
	for (link = n->inlink; link != NULL; link = link->inlink)
		rm_link(link);
	for (link = n->outlink; link != NULL; link = link->outlink)
		rm_link(link);
	delete_node(n);
	modified();
	free_node(n);
}

remove_node(x, y)
int	x, y; 
{
	struct g_node	*node;

	node = node_search(x, y, NODE_TOL);
	if (node == NULL) {
	    put_text("Node not found!");
	    return;
	    }
	rm_node(node);
	}
	
