#include <stdio.h>
#include <xview/xview.h>
#include <sys/stat.h>
#include <errno.h>
#include "tool.h"
#include "g.h"
#include "menu.h"
#include "ext.h"
#include "extxv.h"


node_label(node)
struct g_node *node;
{
	int i;
	char buf[NODE_LABEL_LENGTH];

	sprintf(buf, "%d", node->no); 
	i = strlen(buf) + 1;  /* 1 for \0 string terminator */
	node->label = (char *) malloc(i*sizeof(char));
	strcpy(node->label, buf);
	write_node_label(node);
}

write_label()
{
	pwtext(cur_x + DX, cur_y + 5 + DY, labelname);

}

remove_label(link)
struct g_link *link;
{
	struct g_label *L;

	L = link->label;
	if (L == 0) return;
	pwtextCLR(L->x + DX, L->y + 5 + DY, L->name);
	delete_label(L);
	modified();
	free_label(L);
}


init_edge_label_reader()
{
	int x, y;
	struct g_rspt *rest_point;

	strcpy(&(labelname[0]), "");
	labelptr = -1;
	put_text_higher("Please enter the edge label;");
	put_text_high("   drag the label with the left button down;");
	put_text("   hit the middle button or return to exit. ");

	/* decide label position */
	if (cur_link->rest_points == 0) {
		x = ( cur_link->src_port.x + cur_link->dst_port.x ) / 2;
		y = ( cur_link->src_port.y + cur_link->dst_port.y ) / 2;
		x += 3;
		y -= 3;
	}
	else { /* put label on the first rest point */
		rest_point = cur_link->rest_points;
		x = rest_point->x + 3;
		y = rest_point->y - 3;
	}
	xv_set(canvas_paint_window(canvas), WIN_CURSOR, label_cursor, 0);
	cur_x = x;
	cur_y = y;
}

clear_requestmsg()
{
	put_text("");
	put_text_high("");
	put_text_higher("");
}

edge_label_reader(ie)
Event *ie;
{
	Font_string_dims dims;
	int i;

	ie->ie_locx = zoom*ie->ie_locx - DX;
	ie->ie_locy = zoom*ie->ie_locy - DY;

	if (win_inputnegevent(ie) ) {
		return;
	}
	if ((ie->ie_code == CNTL_H) || (ie->ie_code == DEL)) 
		if (labelptr == -1)  return;
		else {
			write_label();
			labelname[labelptr--] = (char) 0;
			write_label();
			return; 
		}
	if (ie->ie_code == CNTL_U) {
		write_label();
		labelname[0] = (char) 0;
		labelptr = -1;
		return;
	}
	if (ie->ie_code == CR || ie->ie_code == MS_MIDDLE) {
		clear_requestmsg();
		reset_canvas_reader();

		(void) xv_get(font, FONT_STRING_DIMS, labelname, &dims);
		if (mode == PLACE_LINK || mode == ENTER_COMMENT) {
			label = (struct g_label  *) malloc(G_LABEL_SIZE);
			label->no = ++(g->label_no);
			modified();
		}
		label->next = 0;
		label->link = (mode == ENTER_COMMENT || mode == CHANGE_COMMENT) ? 0 : cur_link;
		label->x = cur_x;
		label->y = cur_y;
		label->w = dims.width;
		label->h = dims.height;
		i = strlen(labelname) + 1;  /* 1 for \0 string terminator */
		label->name = (char *) malloc(i*sizeof(char));
		strcpy(label->name, labelname);
		if (mode != ENTER_COMMENT && mode != CHANGE_COMMENT) cur_link->label = label;
		insert_label(label);
		if (mode == PLACE_LINK) {
			put_text("Please point out the source node of the edge.");
		}
		else if (mode == CHANGE_LINK_LABEL) {
			put_text("Please point to the edge label to be changed.");
		} else if (mode == ENTER_COMMENT) {
			put_text("Please point to the position of the new comment.");
		} else put_text("Please point to the comment to be changed.");
		xv_set(canvas_paint_window(canvas), WIN_CURSOR, link_cursor, 0);
		return;
	}	
	if (labelptr >= LABELNAME_LENGTH) return;
	if ((ie->ie_code > 31) && (ie->ie_code < 127)) {
		write_label();
		labelname[++labelptr] = (char) ie->ie_code;
		labelname[labelptr+1] = (char) 0;
		write_label();
		return;
	}	
	switch (ie->ie_code) {
	case	MS_LEFT:
	case LOC_MOVEWHILEBUTDOWN:
		write_label();
		cur_x = ie->ie_locx;
		cur_y = ie->ie_locy;
		write_label();
		return;
	case LOC_WINEXIT:
		return;
	}
}

insert_label(la)
struct g_label *la;
{
	register int 	x, y;
	struct g_label  *q, *r;

	x = la->x;
	y = la->y;
	la->next = g->first_label;
	g->first_label = la;
	modified();
}


delete_label(n)
struct g_label	*n;
{
	struct g_label	*q, *r;

	if (g->first_label == 0) return;
	if (n == 0) return;

	for (r = g->first_label, q = 0; r != 0; q = r, r = r->next) {
		if (r == n) {
			modified();
			if (r == g->first_label)
				g->first_label = n->next;
			else
				q->next = n->next;
			break;
		}
	}
}

free_label(L)
struct g_label *L;
{
	if (L == 0) return;
	if (L->name != 0) free(L->name);
	free(L);
}

struct g_label *
label_search(x,y)
int x, y;
{
	struct g_label *p;
	for (p = g->first_label; p != 0; p = p->next) 
		if (x >= p->x - 7 && x <= p->x + p->w + 6 &&
		    y >= p->y - 7 && y <= p->y + p->h + 6)
			return(p);
	return(0);
}



